<?php
/**
 * DBCart 商城系统
 *
 * ==========================================================================
 * @link      https://www.dbcart.cn/
 * @copyright 北京珑大钜商科技有限公司，并保留所有权利。
 * @license   https://www.dbcart.cn/license.html License
 * ==========================================================================
 *
 * @author    静静的风 <baron@loongdom.cn>
 *
 */

namespace Extend\Controller;

use Admin\Controller\AdminActionController;
use Doctrine\ORM\EntityManager;
use Extend\Entity\Navigation;
use Extend\Entity\NavigationContent;
use Extend\Form\NavigationForm;
use Extend\Service\NavigationContentManager;
use Extend\Service\NavigationManager;
use Laminas\Mvc\I18n\Translator;

class NavigationController extends AdminActionController
{
    private $translator;
    private $entityManager;
    private $navigationManager;
    private $navigationContentManager;

    public function __construct(
        Translator $translator,
        EntityManager $entityManager,
        NavigationManager $navigationManager,
        NavigationContentManager $navigationContentManager
    )
    {
        $this->translator       = $translator;
        $this->entityManager    = $entityManager;
        $this->navigationManager= $navigationManager;
        $this->navigationContentManager = $navigationContentManager;
    }

    /**
     * 导航列表
     * @return array|\Laminas\View\Model\ViewModel
     */
    public function indexAction()
    {
        $navigationList = $this->entityManager->getRepository(Navigation::class)->findAllNavigation();

        return ['navigationList' => $navigationList];
    }

    /**
     * 添加导航
     * @return NavigationForm[]|\Laminas\Http\Response
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    public function addAction()
    {
        $form = new NavigationForm();
        $form->get('navigationPlace')->setValueOptions([1 => $this->translator->translate('顶部'), 2 => $this->translator->translate('中部'), 3 => $this->translator->translate('底部')]);
        $form->get('goodsClassId')->setValueOptions($this->goodsPlugin()->classListOptions($this->translator->translate('选择商品分类')));

        if($this->getRequest()->isPost()) {
            $data = $this->params()->fromPost();
            $form->setData($data);
            if ($form->isValid()) {
                $data = $form->getData();
                $navigation = $this->navigationManager->addNavigation($data);
                $this->navigationContentManager->addOrEditNavigationContent($data, $navigation->getNavigationId());

                $this->adminCommon()->addOperLog(sprintf($this->translator->translate('导航 %s 添加成功!'), $data['navigationTitle'.SHOP_FONT_LANGUAGE]), $this->translator->translate('导航设置'));

                return $this->redirect()->toRoute('navigation');
            }
        }

        return ['form' => $form];
    }

    /**
     * 编辑导航
     * @return array|\Laminas\Http\Response
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    public function editAction()
    {
        $navigationId = (int) $this->params()->fromRoute('id', -1);

        $navigation   = $this->entityManager->getRepository(Navigation::class)->findOneBy(['navigationId' => $navigationId]);
        if ($navigation == null) {
            $this->flashMessenger()->addWarningMessage($this->translator->translate('该导航不存在!'));
            return $this->redirect()->toRoute('navigation');
        }

        $form = new NavigationForm();
        $form->get('navigationPlace')->setValueOptions([1 => $this->translator->translate('顶部'), 2 => $this->translator->translate('中部'), 3 => $this->translator->translate('底部')]);
        $form->get('goodsClassId')->setValueOptions($this->goodsPlugin()->classListOptions($this->translator->translate('选择商品分类')));

        if($this->getRequest()->isPost()) {
            $data = $this->params()->fromPost();
            $form->setData($data);
            if ($form->isValid()) {
                $data = $form->getData();
                $this->navigationManager->editNavigation($data, $navigation);
                $this->navigationContentManager->addOrEditNavigationContent($data, $navigationId);

                $this->adminCommon()->addOperLog(sprintf($this->translator->translate('导航 %s 编辑成功!'), $data['navigationTitle'.SHOP_FONT_LANGUAGE]), $this->translator->translate('导航设置'));

                return $this->redirect()->toRoute('navigation');
            }
        } else {
            $form->setData($navigation->valuesArray());

            $navigationContent = $this->entityManager->getRepository(NavigationContent::class)->findBy(['navigationId' => $navigationId]);
            foreach ($navigationContent as $contentValue) {
                if (!in_array($contentValue->getLanguageCode(), SHOP_LANGUAGE)) continue;
                $form->setData(['navigationTitle'.$contentValue->getLanguageCode() => $contentValue->getNavigationTitle()]);
            }
        }

        return ['form' => $form, 'navigation' => $navigation];
    }

    /**
     * 删除导航
     * @return \Laminas\Http\Response
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    public function deleteAction()
    {
        if(!$this->adminCommon()->validatorCsrf()) return $this->adminCommon()->toReferer();

        $navigationId = (int) $this->params()->fromRoute('id', -1);
        $navigation   = $this->entityManager->getRepository(Navigation::class)->findOneBy(['navigationId' => $navigationId]);
        if ($navigation == null) {
            $this->flashMessenger()->addWarningMessage($this->translator->translate('该导航不存在!'));
            return $this->redirect()->toRoute('navigation');
        }

        $navigationContent = $this->entityManager->getRepository(NavigationContent::class)->findOneBy(['navigationId' => $navigationId, 'languageCode' => SHOP_FONT_LANGUAGE]);

        $this->navigationManager->deleteNavigation($navigation);
        $this->navigationContentManager->deleteNavigationContent($navigationId);

        $this->adminCommon()->addOperLog(sprintf($this->translator->translate('导航 %s 删除成功!'), $navigationContent->getNavigationTitle()), $this->translator->translate('导航设置'));

        return $this->adminCommon()->toReferer();
    }
}